@@ -20,7 +20,8 @@
 # csh original by James Woods; sh conversion by David MacKenzie.
 
 usage="\
-Usage: updatedb [--localpaths='dir1 dir2...'] [--netpaths='dir1 dir2...']
+Usage: updatedb [--findoptions='-option1 -option2...']
+       [--localpaths='dir1 dir2...'] [--netpaths='dir1 dir2...']
        [--prunepaths='dir1 dir2...'] [--prunefs='fs1 fs2...']
        [--output=dbfile] [--netuser=user] [--localuser=user] 
        [--old-format] [--version] [--help]
@@ -38,6 +39,7 @@ do
   opt=`echo $arg|sed 's/^\([^=]*\).*/\1/'`  || exit 71
   val=`echo $arg|sed 's/^[^=]*=\(.*\)/\1/'` || exit 71
   case "$opt" in
+    --findoptions) FINDOPTIONS="$val" ;;
     --localpaths) SEARCHPATHS="$val" ;;
     --netpaths) NETPATHS="$val" ;;
     --prunepaths) PRUNEPATHS="$val" ;;
@@ -55,6 +57,25 @@ $usage" >&2
   esac
 done
 
+if test "$old" = yes; then
+    echo "Warning: future versions of findutils will shortly discontinue support for the old locate database format." >&2
+
+    sort="@SORT@"
+    print_option="-print"
+    frcode_options=""
+else
+    if @SORT_SUPPORTS_Z@
+    then
+        sort="@SORT@ -z"
+        print_option="-print0"
+        frcode_options="-0"
+    else
+        sort="@SORT@"
+        print_option="-print"
+        frcode_options=""
+    fi
+fi
+
 getuid() {
     # format of "id" output is ...
     # uid=1(daemon) gid=1(other)
@@ -65,6 +86,12 @@ getuid() {
 # You can set these in the environment, or use command-line options,
 # to override their defaults:
 
+# Any global options for find?
+: ${FINDOPTIONS=}
+
+# What shell shoud we use?  We should use a POSIX-ish sh.
+: ${SHELL="/bin/sh"}
+
 # Non-network directories to put in the database.
 : ${SEARCHPATHS="/"}
 
@@ -72,7 +99,7 @@ getuid() {
 : ${NETPATHS=}
 
 # Directories to not put in the database, which would otherwise be.
-: ${PRUNEPATHS="/tmp /usr/tmp /var/tmp /afs"}
+: ${PRUNEPATHS="/tmp /usr/tmp /var/tmp /afs /amd /sfs"}
 
 # The same, in the form of a regex that find can use.
 test -z "$PRUNEREGEX" &&
@@ -114,9 +141,10 @@ fi
 : ${bigram:=${LIBEXECDIR}/@bigram@}
 : ${code:=${LIBEXECDIR}/@code@}
 
+
 PATH=/bin:/usr/bin:${BINDIR}; export PATH
 
-: ${PRUNEFS="nfs NFS proc"}
+: ${PRUNEFS="nfs NFS proc afs proc smbfs autofs iso9660 ncpfs coda devpts ftpfs devfs mfs sysfs shfs"}
 
 if test -n "$PRUNEFS"; then
 prunefs_exp=`echo $PRUNEFS |sed -e 's/\([^ ][^ ]*\)/-o -fstype \1/g' \
@@ -139,15 +167,15 @@ cd "$changeto"
 if test -n "$SEARCHPATHS"; then
   if [ "$LOCALUSER" != "" ]; then
     # : A1
-    su $LOCALUSER -c \
-    "$find $SEARCHPATHS \
+    su $LOCALUSER -s $SHELL -c \
+    "$find $SEARCHPATHS $FINDOPTIONS \
      \\( $prunefs_exp \
-     -type d -regex '$PRUNEREGEX' \\) -prune -o -print"
+     -type d -regex '$PRUNEREGEX' \\) -prune -o $print_option"
   else
     # : A2
-    $find $SEARCHPATHS \
+    $find $SEARCHPATHS $FINDOPTIONS \
      \( $prunefs_exp \
-     -type d -regex "$PRUNEREGEX" \) -prune -o -print
+     -type d -regex "$PRUNEREGEX" \) -prune -o $print_option
   fi
 fi
 
@@ -155,16 +183,16 @@ if test -n "$NETPATHS"; then
 myuid=`getuid` 
 if [ "$myuid" = 0 ]; then
     # : A3
-    su $NETUSER -c \
-     "$find $NETPATHS \\( -type d -regex '$PRUNEREGEX' -prune \\) -o -print" ||
+    su $NETUSER -s $SHELL -c \
+     "$find $NETPATHS $FINDOPTIONS \\( -type d -regex '$PRUNEREGEX' -prune \\) -o $print_option" ||
     exit $?
   else
     # : A4
-    $find $NETPATHS \( -type d -regex "$PRUNEREGEX" -prune \) -o -print ||
+    $find $NETPATHS $FINDOPTIONS \( -type d -regex "$PRUNEREGEX" -prune \) -o $print_option ||
     exit $?
   fi
 fi
-} | sort -f | $frcode > $LOCATE_DB.n
+} | $sort -f | $frcode $frcode_options > $LOCATE_DB.n
 then
     # OK so far
     true
@@ -209,15 +237,15 @@ cd "$changeto"
 if test -n "$SEARCHPATHS"; then
   if [ "$LOCALUSER" != "" ]; then
     # : A5
-    su $LOCALUSER -c \
-    "$find $SEARCHPATHS \
+    su $LOCALUSER -s $SHELL -c \
+    "$find $SEARCHPATHS $FINDOPTIONS \
      \( $prunefs_exp \
-     -type d -regex '$PRUNEREGEX' \) -prune -o -print" || exit $?
+     -type d -regex '$PRUNEREGEX' \) -prune -o $print_option" || exit $?
   else
     # : A6
-    $find $SEARCHPATHS \
+    $find $SEARCHPATHS $FINDOPTIONS \
      \( $prunefs_exp \
-     -type d -regex "$PRUNEREGEX" \) -prune -o -print || exit $?
+     -type d -regex "$PRUNEREGEX" \) -prune -o $print_option || exit $?
   fi
 fi
 
@@ -225,19 +253,19 @@ if test -n "$NETPATHS"; then
   myuid=`getuid`
   if [ "$myuid" = 0 ]; then
     # : A7
-    su $NETUSER -c \
-     "$find $NETPATHS \\( -type d -regex '$PRUNEREGEX' -prune \\) -o -print" ||
+    su $NETUSER -s $SHELL -c \
+     "$find $NETPATHS $FINDOPTIONS \\( -type d -regex '$PRUNEREGEX' -prune \\) -o $print_option" ||
     exit $?
   else
     # : A8
-    $find $NETPATHS \( -type d -regex "$PRUNEREGEX" -prune \) -o -print ||
+    $find $NETPATHS $FINDOPTIONS \( -type d -regex "$PRUNEREGEX" -prune \) -o $print_option ||
     exit $?
   fi
 fi
-} | tr / '\001' | sort -f | tr '\001' / > $filelist
+} | tr / '\001' | $sort -f | tr '\001' / > $filelist
 
 # Compute the (at most 128) most common bigrams in the file list.
-$bigram < $filelist | sort | uniq -c | sort -nr |
+$bigram $bigram_opts < $filelist | sort | uniq -c | sort -nr |
   awk '{ if (NR <= 128) print $2 }' | tr -d '\012' > $bigrams
 
 # Code the file list.
